// ble_uart_controller.dart

import 'package:example/controllers/ble_servicescontroller.dart';
import 'package:flutter_blue_plus/flutter_blue_plus.dart';
import 'package:get/get.dart';



/// Controller for managing Bluetooth Low Energy (BLE) UART functionality.
class BleUartController extends GetxController {
  /// UUID for the BLE service used in UART communication.
  final String serviceUuid = '6e400001-b5a3-f393-e0a9-e50e24dcca9e';

  /// UUID for the characteristic used to send commands.
  final String sendCharacteristicUuid = '6e400002-b5a3-f393-e0a9-e50e24dcca9e';

  /// UUID for the characteristic used to receive commands.
  final String receiveCharacteristicUuid = '6e400003-b5a3-f393-e0a9-e50e24dcca9e';

  /// Observable string for storing received data.
  RxString receivingValue = "".obs;

  /// Currently connected Bluetooth device.
  BluetoothDevice? device;

  /// Bluetooth service for UART communication.
  late BluetoothService service;

  /// Characteristic for sending commands.
  late BluetoothCharacteristic sendCharacteristic;

  /// Characteristic for receiving commands.
  late BluetoothCharacteristic receiveCharacteristic;

  /// Reference to the BleDeviceController for accessing connected device information.
  BleDeviceController bleDeviceController = Get.find();

  /// Send a command through the BLE UART communication.
  void sendCommand(String command) async {
    // Get connected device
    device = bleDeviceController.connectedDevice;

    // Discover services
    List<BluetoothService> services = await device!.discoverServices();
    for (var serv in services) {
      if (serv.uuid.toString() == serviceUuid) {
        service = serv;
      }
    }

    // Find send characteristic
    for (var char in service.characteristics) {
      if (char.uuid.toString() == sendCharacteristicUuid) {
        sendCharacteristic = char;
      }
    }

    // Check if service and characteristic are found
    if (service == null || sendCharacteristic == null) {
      print('Service or characteristic not found.');
      return;
    }

    // Append CR and LF characters to the command
    String commandWithCRLF = '$command${String.fromCharCode(13)}${String.fromCharCode(10)}';

    List<int> bytes = commandWithCRLF.codeUnits;
    await sendCharacteristic.write(bytes);
    print("Data sent: $commandWithCRLF");

    // Receive the response
    receiveCommand();
  }

  /// Receive a command through the BLE UART communication.
  void receiveCommand() async {
    // Get connected device
    device = bleDeviceController.connectedDevice;

    // Discover services
    List<BluetoothService> services = await device!.discoverServices();
    for (var serv in services) {
      if (serv.uuid.toString() == serviceUuid) {
        service = serv;
      }
    }

    // Find receive characteristic
    for (var char in service.characteristics) {
      if (char.uuid.toString() == receiveCharacteristicUuid) {
        receiveCharacteristic = char;
      }
    }

    // Check if service and characteristic are found
    if (service == null || receiveCharacteristic == null) {
      print('Service or characteristic not found.');
      return;
    }

    // Set notify value to true and listen for data
    receiveCharacteristic.setNotifyValue(true);
    receiveCharacteristic.value.listen((value) {
      String receivedData = String.fromCharCodes(value);
      receivingValue.value = receivedData;
      print('Received data: $receivedData');
    });
  }

  /// Initialize the BLE UART controller.
  void init() async {
    // Get connected device
    device = Get.find<BleDeviceController>().connectedDevice!;

    // Discover services
    List<BluetoothService> services = await device!.discoverServices();
    for (var serv in services) {
      if (serv.uuid.toString() == serviceUuid) {
        service = serv;
      }
    }

    // Check if service is found
    if (service == null) {
      print('Service with UUID $serviceUuid not found.');
      return;
    }

    // Find send characteristic
    sendCharacteristic = service.characteristics.firstWhere((c) => c.uuid.toString() == sendCharacteristicUuid);

    // Check if characteristic is found
    if (sendCharacteristic == null || receiveCharacteristic == null) {
      print('Characteristics not found.');
      return;
    }
  }

  /// Dispose of resources when the controller is no longer needed.
  void dispose() {
    sendCharacteristic.setNotifyValue(false);
  }
}
